package de.docutain.docutain_sdk

import android.app.Activity
import android.content.Context
import android.content.Intent
import androidx.annotation.NonNull
import android.util.Log

import org.json.JSONObject

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.PluginRegistry

import de.docutain.sdk.DocutainSDK.pageCount
import de.docutain.sdk.DocutainSDK.initSDK
import de.docutain.sdk.DocutainSDK.getLastError
import de.docutain.sdk.ui.DocumentScanner
import de.docutain.sdk.Document
import de.docutain.sdk.Logger
import de.docutain.sdk.ui.DocutainColor
import de.docutain.sdk.dataextraction.AnalyzeConfiguration
import de.docutain.sdk.dataextraction.DocumentDataReader

import java.io.File

/** DocutainSdkPlugin */
class DocutainSdkPlugin: FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.ActivityResultListener {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var context : Context
  private var act: android.app.Activity? = null
  private lateinit var result: Result
  val SCAN_DOCUMENT_RESULT_CODE = 1

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    context = flutterPluginBinding.applicationContext
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "docutain_sdk")
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    this.result = result //saves the result to call it when the user
    if (call.method == "initSDK") {
      val licenseKey : String? = call.argument("licenseKey");
      val rc = initSDK(context as android.app.Application, licenseKey!!)
      result.success(rc)
    }  else if (call.method == "getLastError") {
      result.success(getLastError())
    } else if (call.method == "getTraceFile") {
      val traceFile = Logger.getTraceFile()
      result.success(traceFile.path)
    }  else if (call.method == "setLogLevel"){
      val level = Logger.Level.values()[call.argument("level")!!]
      Logger.setLogLevel(level)
      result.success(true)
    } else if (call.method == "scanDocument") {
      if(act != null){
        val nativeScanConfig = de.docutain.sdk.ui.DocumentScannerConfiguration()
        nativeScanConfig.allowCaptureModeSetting = call.argument("allowCaptureModeSetting")!!
        nativeScanConfig.pageEditConfig.allowPageRotation = call.argument("allowPageRotation")!!
        nativeScanConfig.pageEditConfig.allowPageFilter  = call.argument("allowPageFilter")!!
        //colors
        val colorPrimaryLight : Long? = call.argument("colorPrimaryLight")
        val colorPrimaryDark : Long? = call.argument("colorPrimaryDark")
        if(colorPrimaryLight != null && colorPrimaryDark != null){
          nativeScanConfig.colorConfig.colorPrimary = DocutainColor(colorPrimaryLight.toHex(), colorPrimaryDark.toHex())
        }
        val colorSecondaryLight : Long? = call.argument("colorSecondaryLight")
        val colorSecondaryDark : Long? = call.argument("colorSecondaryDark")
        if(colorSecondaryLight != null && colorSecondaryDark != null){
          nativeScanConfig.colorConfig.colorSecondary = DocutainColor(colorSecondaryLight.toHex(), colorSecondaryDark.toHex())
        }
        val colorOnSecondaryLight : Long? = call.argument("colorOnSecondaryLight")
        val colorOnSecondaryDark : Long? = call.argument("colorOnSecondaryDark")
        if(colorOnSecondaryLight != null && colorOnSecondaryDark != null){
          nativeScanConfig.colorConfig.colorOnSecondary = DocutainColor(colorOnSecondaryLight.toHex(), colorOnSecondaryDark.toHex())
        }
        val colorScanButtonsLayoutBackgroundLight : Long? = call.argument("colorScanButtonsLayoutBackgroundLight")
        val colorScanButtonsLayoutBackgroundDark : Long? = call.argument("colorScanButtonsLayoutBackgroundDark")
        if(colorScanButtonsLayoutBackgroundLight != null && colorScanButtonsLayoutBackgroundDark != null){
          nativeScanConfig.colorConfig.colorScanButtonsLayoutBackground = DocutainColor(colorScanButtonsLayoutBackgroundLight.toHex(), colorScanButtonsLayoutBackgroundDark.toHex())
        }
        val colorScanButtonsForegroundLight : Long? = call.argument("colorScanButtonsForegroundLight")
        val colorScanButtonsForegroundDark : Long? = call.argument("colorScanButtonsForegroundDark")
        if(colorScanButtonsForegroundLight != null && colorScanButtonsForegroundDark != null){
          nativeScanConfig.colorConfig.colorScanButtonsForeground = DocutainColor(colorScanButtonsForegroundLight.toHex(), colorScanButtonsForegroundDark.toHex())
        }
        val colorScanPolygonLight : Long? = call.argument("colorScanPolygonLight")
        val colorScanPolygonDark : Long? = call.argument("colorScanPolygonDark")
        if(colorScanPolygonLight != null && colorScanPolygonDark != null){
          nativeScanConfig.colorConfig.colorScanPolygon = DocutainColor(colorScanPolygonLight.toHex(), colorScanPolygonDark.toHex())
        }
        val colorBottomBarBackgroundLight : Long? = call.argument("colorBottomBarBackgroundLight")
        val colorBottomBarBackgroundDark : Long? = call.argument("colorBottomBarBackgroundDark")
        if(colorBottomBarBackgroundLight != null && colorBottomBarBackgroundDark != null){
          nativeScanConfig.colorConfig.colorBottomBarBackground = DocutainColor(colorBottomBarBackgroundLight.toHex(), colorBottomBarBackgroundDark.toHex())
        }
        val colorBottomBarForegroundLight : Long? = call.argument("colorBottomBarForegroundLight")
        val colorBottomBarForegroundDark : Long? = call.argument("colorBottomBarForegroundDark")
        if(colorBottomBarForegroundLight != null && colorBottomBarForegroundDark != null){
          nativeScanConfig.colorConfig.colorBottomBarForeground = DocutainColor(colorBottomBarForegroundLight.toHex(), colorBottomBarForegroundDark.toHex())
        }
        val colorTopBarBackgroundLight : Long? = call.argument("colorTopBarBackgroundLight")
        val colorTopBarBackgroundDark : Long? = call.argument("colorTopBarBackgroundDark")
        if(colorTopBarBackgroundLight != null && colorTopBarBackgroundDark != null){
          nativeScanConfig.colorConfig.colorTopBarBackground = DocutainColor(colorTopBarBackgroundLight.toHex(), colorTopBarBackgroundDark.toHex())
        }
        val colorTopBarForegroundLight : Long? = call.argument("colorTopBarForegroundLight")
        val colorTopBarForegroundDark : Long? = call.argument("colorTopBarForegroundDark")
        if(colorTopBarForegroundLight != null && colorTopBarForegroundDark != null){
          nativeScanConfig.colorConfig.colorTopBarForeground = DocutainColor(colorTopBarForegroundLight.toHex(), colorTopBarForegroundDark.toHex())
        }
        //start scan activity
        val intent = DocumentScanner.newScanIntent(context, nativeScanConfig)
        act!!.startActivityForResult(intent, SCAN_DOCUMENT_RESULT_CODE)
      } else{
        result.success(false)
      }
    }else if (call.method == "writePDF") {
      val path : String? = call.argument("path");
      val fileName : String? = call.argument("fileName");
      val overwrite : Boolean? = call.argument("overwrite");
      val pdfPageFormat = Document.PDFPageFormat.values()[call.argument("pdfPageFormat")!!];
      val returnFile = Document.writePDF(java.io.File(path!!, fileName!!), overwrite!!, pdfPageFormat);
      if(returnFile != null){
        return result.success(returnFile.parent!! + "/" + returnFile.name);
      } else{
        return result.success(null);
      }
    } else if (call.method == "setAnalyzeConfiguration"){
      val analyzeConfig = AnalyzeConfiguration()
      analyzeConfig.readPaymentState = call.argument("readPaymentState")!!
      analyzeConfig.readBIC = call.argument("readBIC")!!
      return result.success(DocumentDataReader.setAnalyzeConfiguration(analyzeConfig))
    } else if (call.method == "analyze"){
      return result.success(DocumentDataReader.analyze())
    } else if (call.method == "getText"){
      val pageNumber : Int = call.argument("pageNumber")!!;
      return result.success(DocumentDataReader.getText(pageNumber))
    }else if (call.method == "pageCount"){
      return result.success(pageCount())
    }
//    else if (call.method == "writeImage") {
//      val path : String? = call.argument("path");
//      val fileName : String? = call.argument("fileName");
//      val page : Int? = call.argument("page");
//      val returnFile = Document.writeImage(page, java.io.File(path!!, fileName!!));
//      if(returnFile != null){
//        return result.success(returnFile.parent!! + "/" + returnFile.name);
//      } else{
//        return result.success(null);
//      }
    //}
  else {
      result.notImplemented()
    }
  }

  private fun Long.toHex() : String{
    return "#${toString(16)}"
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }
  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    act = binding.activity
    binding.addActivityResultListener(this)
  }
  override fun onDetachedFromActivityForConfigChanges() {
    act = null
  }
  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    act = binding.activity
    binding.addActivityResultListener(this)
  }
  override fun onDetachedFromActivity() {
    act = null
  }
  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    if (requestCode == SCAN_DOCUMENT_RESULT_CODE) {
      result.success(if (resultCode == Activity.RESULT_OK) true else false)
      return true
    }
    return false
  }
}
