import Flutter
import UIKit
import DocutainSdk

public class DocutainSdkPlugin: NSObject, FlutterPlugin {

  private var result : FlutterResult? = nil

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "docutain_sdk", binaryMessenger: registrar.messenger())
    let instance = DocutainSdkPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "initSDK":
      let args = call.arguments as! Dictionary<String, Any>
      let licenseKey = args["licenseKey"] as! String
      result(DocutainSDK.initSDK(licenseKey: licenseKey))
    case "getLastError":
      result(DocutainSDK.getLastError())
    case "getTraceFile":
      result(Logger.getTraceFile())
    case "setLogLevel":
      let args = call.arguments as! Dictionary<String, Any>
      let levelValue = args["level"] as! UInt32
      let level = Logger.Level(rawValue: levelValue)
      Logger.setLogLevel(level:level!)
      result(true)
    case "scanDocument":
      let scanConfig = DocumentScannerConfiguration()
      let args = call.arguments as! Dictionary<String, Any>
      scanConfig.allowCaptureModeSetting = args["allowCaptureModeSetting"] as! Bool
      scanConfig.pageEditConfig.allowPageFilter = args["allowPageFilter"] as! Bool
      scanConfig.pageEditConfig.allowPageRotation = args["allowPageRotation"] as! Bool
      //colors
      let colorPrimaryLight = args["colorPrimaryLight"] as? Int
      let colorPrimaryDark = args["colorPrimaryDark"] as? Int
      if(colorPrimaryLight != nil && colorPrimaryDark != nil){
        scanConfig.colorConfig.setColorPrimary(light: UIColorFromRGB(colorPrimaryLight!), dark: UIColorFromRGB(colorPrimaryDark!))
      }
      let colorSecondaryLight = args["colorSecondaryLight"] as? Int
      let colorSecondaryDark = args["colorSecondaryDark"] as? Int
      if(colorSecondaryLight != nil && colorSecondaryDark != nil){
        scanConfig.colorConfig.setColorSecondary(light: UIColorFromRGB(colorSecondaryLight!), dark: UIColorFromRGB(colorSecondaryDark!))
      }
      let colorOnSecondaryLight = args["colorOnSecondaryLight"] as? Int
      let colorOnSecondaryDark = args["colorOnSecondaryDark"] as? Int
      if(colorOnSecondaryLight != nil && colorOnSecondaryDark != nil){
        scanConfig.colorConfig.setColorOnSecondary(light: UIColorFromRGB(colorOnSecondaryLight!), dark: UIColorFromRGB(colorOnSecondaryDark!))
      }
      let colorScanButtonsLayoutBackgroundLight = args["colorScanButtonsLayoutBackgroundLight"] as? Int
      let colorScanButtonsLayoutBackgroundDark = args["colorScanButtonsLayoutBackgroundDark"] as? Int
      if(colorScanButtonsLayoutBackgroundLight != nil && colorScanButtonsLayoutBackgroundDark != nil){
        scanConfig.colorConfig.setColorScanButtonsLayoutBackground(light: UIColorFromRGB(colorScanButtonsLayoutBackgroundLight!), dark: UIColorFromRGB(colorScanButtonsLayoutBackgroundDark!))
      }
      let colorScanButtonsForegroundLight = args["colorScanButtonsForegroundLight"] as? Int
      let colorScanButtonsForegroundDark = args["colorScanButtonsForegroundDark"] as? Int
      if(colorScanButtonsForegroundLight != nil && colorScanButtonsForegroundDark != nil){
        scanConfig.colorConfig.setColorScanButtonsForeground(light: UIColorFromRGB(colorScanButtonsForegroundLight!), dark: UIColorFromRGB(colorScanButtonsForegroundDark!))
      }
      let colorScanPolygonLight = args["colorScanPolygonLight"] as? Int
      let colorScanPolygonDark = args["colorScanPolygonDark"] as? Int
      if(colorScanPolygonLight != nil && colorScanPolygonDark != nil){
        scanConfig.colorConfig.setColorScanPolygon(light: UIColorFromRGB(colorScanPolygonLight!), dark: UIColorFromRGB(colorScanPolygonDark!))
      }
      let colorBottomBarBackgroundLight = args["colorBottomBarBackgroundLight"] as? Int
      let colorBottomBarBackgroundDark = args["colorBottomBarBackgroundDark"] as? Int
      if(colorBottomBarBackgroundLight != nil && colorBottomBarBackgroundDark != nil){
        scanConfig.colorConfig.setColorBottomBarBackground(light: UIColorFromRGB(colorBottomBarBackgroundLight!), dark: UIColorFromRGB(colorBottomBarBackgroundDark!))
      }
      let colorBottomBarForegroundLight = args["colorBottomBarForegroundLight"] as? Int
      let colorBottomBarForegroundDark = args["colorBottomBarForegroundDark"] as? Int
      if(colorBottomBarForegroundLight != nil && colorBottomBarForegroundDark != nil){
        scanConfig.colorConfig.setColorBottomBarForeground(light: UIColorFromRGB(colorBottomBarForegroundLight!), dark: UIColorFromRGB(colorBottomBarForegroundDark!))
      }
      let colorTopBarBackgroundLight = args["colorTopBarBackgroundLight"] as? Int
      let colorTopBarBackgroundDark = args["colorTopBarBackgroundDark"] as? Int
      if(colorTopBarBackgroundLight != nil && colorTopBarBackgroundDark != nil){
        scanConfig.colorConfig.setColorTopBarBackground(light: UIColorFromRGB(colorTopBarBackgroundLight!), dark: UIColorFromRGB(colorTopBarBackgroundDark!))
      }
      let colorTopBarForegroundLight = args["colorTopBarForegroundLight"] as? Int
      let colorTopBarForegroundDark = args["colorTopBarForegroundDark"] as? Int
      if(colorTopBarForegroundLight != nil && colorTopBarForegroundDark != nil){
        scanConfig.colorConfig.setColorTopBarForeground(light: UIColorFromRGB(colorTopBarForegroundLight!), dark: UIColorFromRGB(colorTopBarForegroundDark!))
      }
      self.result = result //need to store to be able to call it in delegate
      UI.scanDocument(scanDelegate: self, scanConfig: scanConfig)
    case "writePDF":
      let args = call.arguments as! Dictionary<String, Any>
      let path = args["path"] as! String
      let fileName = args["fileName"] as! String
      let overWrite = args["overwrite"] as! Bool
      let pdfFormatValue = args["pdfPageFormat"] as! Int
      let pdfPageFormat = Document.PDFPageFormat(rawValue: pdfFormatValue)
      result(Document.writePDF(fileUrl: URL(string: path)!, fileName: fileName, overwrite: overWrite, pageFormat: pdfPageFormat!)?.path)
    case "setAnalyzeConfiguration":
      if #available(iOS 13, *) {
        let args = call.arguments as! Dictionary<String, Any>
        let analyzeConfig = AnalyzeConfiguration()
        analyzeConfig.readBIC = args["readBIC"] as! Bool
        analyzeConfig.readPaymentState = args["readPaymentState"] as! Bool
        result(DocumentDataReader.setAnalyzeConfiguration(analyzeConfiguration: analyzeConfig))
      } else {
          result(false)
      }
    case "analyze":
     if #available(iOS 13, *) {
      result(DocumentDataReader.analyze())
     } else {
      result("")
     }
    case "getText":
      if #available(iOS 13, *) {
        let args = call.arguments as! Dictionary<String, Any>
        let pageNumber = args["pageNumber"] as! Int32
        result(DocumentDataReader.getText(pageNumber: pageNumber))
      } else {
        result("")
      }
    case "pageCount":
      result(DocutainSDK.pageCount())
    default:
      result(FlutterMethodNotImplemented)
    }
  }

  private func UIColorFromRGB(_ rgbValue: Int) -> UIColor! {
    return UIColor(
        red: CGFloat((Float((rgbValue & 0xff0000) >> 16)) / 255.0),
        green: CGFloat((Float((rgbValue & 0x00ff00) >> 8)) / 255.0),
        blue: CGFloat((Float((rgbValue & 0x0000ff) >> 0)) / 255.0),
        alpha: 1.0)
  }
}

// These should be called but are not
extension DocutainSdkPlugin: ScanDelegate {
    public func didFinishScan(withResult result: Bool) {
      self.result!(result)
      self.result = nil //reset
    }
}
